@@ -76,7 +76,12 @@ options:
     value:
         description:
             - Sets the record value.
-
+    protocol:
+        description:
+            - Sets the transport protocol (TCP or UDP). TCP is the recommended and a more robust option.
+        default: 'tcp'
+        choices: ['tcp', 'udp']
+        version_added: 2.8
 '''
 
 EXAMPLES = '''
@@ -113,11 +118,11 @@ RETURN = '''
 changed:
     description: If module has modified record
     returned: success
-    type: string
+    type: str
 record:
     description: DNS record
     returned: success
-    type: string
+    type: str
     sample: 'ansible'
 ttl:
     description: DNS record TTL
@@ -127,7 +132,7 @@ ttl:
 type:
     description: DNS record type
     returned: success
-    type: string
+    type: str
     sample: 'CNAME'
 value:
     description: DNS record value(s)
@@ -137,7 +142,7 @@ value:
 zone:
     description: DNS record zone
     returned: success
-    type: string
+    type: str
     sample: 'example.org.'
 dns_rc:
     description: dnspython return code
@@ -147,13 +152,16 @@ dns_rc:
 dns_rc_str:
     description: dnspython return code (string representation)
     returned: always
-    type: string
+    type: str
     sample: 'REFUSED'
 '''
 
+import traceback
+
 from binascii import Error as binascii_error
 from socket import error as socket_error
 
+DNSPYTHON_IMP_ERR = None
 try:
     import dns.update
     import dns.query
@@ -163,9 +171,10 @@ try:
 
     HAVE_DNSPYTHON = True
 except ImportError:
+    DNSPYTHON_IMP_ERR = traceback.format_exc()
     HAVE_DNSPYTHON = False
 
-from ansible.module_utils.basic import AnsibleModule
+from ansible.module_utils.basic import AnsibleModule, missing_required_lib
 from ansible.module_utils._text import to_native
 
 
@@ -227,7 +236,10 @@ class RecordManager(object):
     def __do_update(self, update):
         response = None
         try:
-            response = dns.query.tcp(update, self.module.params['server'], timeout=10, port=self.module.params['port'])
+            if self.module.params['protocol'] == 'tcp':
+                response = dns.query.tcp(update, self.module.params['server'], timeout=10, port=self.module.params['port'])
+            else:
+                response = dns.query.udp(update, self.module.params['server'], timeout=10, port=self.module.params['port'])
         except (dns.tsig.PeerBadKey, dns.tsig.PeerBadSignature) as e:
             self.module.fail_json(msg='TSIG update error (%s): %s' % (e.__class__.__name__, to_native(e)))
         except (socket_error, dns.exception.Timeout) as e:
@@ -354,7 +366,10 @@ class RecordManager(object):
         query = dns.message.make_query(self.fqdn, self.module.params['type'])
 
         try:
-            lookup = dns.query.tcp(query, self.module.params['server'], timeout=10, port=self.module.params['port'])
+            if self.module.params['protocol'] == 'tcp':
+                lookup = dns.query.tcp(query, self.module.params['server'], timeout=10, port=self.module.params['port'])
+            else:
+                lookup = dns.query.udp(query, self.module.params['server'], timeout=10, port=self.module.params['port'])
         except (socket_error, dns.exception.Timeout) as e:
             self.module.fail_json(msg='DNS server error: (%s): %s' % (e.__class__.__name__, to_native(e)))
 
@@ -378,13 +393,14 @@ def main():
             record=dict(required=True, type='str'),
             type=dict(required=False, default='A', type='str'),
             ttl=dict(required=False, default=3600, type='int'),
-            value=dict(required=False, default=None, type='list')
+            value=dict(required=False, default=None, type='list'),
+            protocol=dict(required=False, default='tcp', choices=['tcp', 'udp'], type='str')
         ),
         supports_check_mode=True
     )
 
     if not HAVE_DNSPYTHON:
-        module.fail_json(msg='python library dnspython required: pip install dnspython')
+        module.fail_json(msg=missing_required_lib('dnspython'), exception=DNSPYTHON_IMP_ERR)
 
     if len(module.params["record"]) == 0:
         module.fail_json(msg='record cannot be empty.')
