@@ -10,7 +10,7 @@ __metaclass__ = type
 
 ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
-                    'supported_by': 'community'}
+                    'supported_by': 'certified'}
 
 DOCUMENTATION = r'''
 ---
@@ -23,11 +23,18 @@ options:
   name:
     description:
       - Specifies the name of the trunk.
+    type: str
     required: True
   interfaces:
     description:
       - The interfaces that are part of the trunk.
       - To clear the list of interfaces, specify an empty list.
+    type: list
+  description:
+    description:
+      - Description of the trunk.
+    type: str
+    version_added: 2.7
   link_selection_policy:
     description:
       - Specifies, once the trunk is configured, the policy that the trunk uses to determine
@@ -38,6 +45,7 @@ options:
         same media type and speed.
       - When C(maximum-bandwidth), specifies that the system determines which interfaces
         can handle new traffic based on the members' maximum bandwidth.
+    type: str
     choices:
       - auto
       - maximum-bandwidth
@@ -54,6 +62,7 @@ options:
         address of the destination.
       - When C(source-destination-ip), specifies that the system bases the hash on the
         combined IP addresses of the source and the destination.
+    type: str
     choices:
       - destination-mac
       - source-destination-ip
@@ -79,6 +88,7 @@ options:
         regardless of whether the partner system has issued a request.
       - When C(passive), specifies that the system sends control packets only when
         the partner system has issued a request.
+    type: str
     choices:
       - active
       - passive
@@ -89,73 +99,114 @@ options:
         C(long).
       - When C(long), specifies that the system sends an LACP control packet every 30 seconds.
       - When C(short), specifies that the system sends an LACP control packet every 1 seconds.
+    type: str
     choices:
       - long
       - short
+  qinq_ethertype:
+    description:
+      - Specifies the ether-type value used for the packets handled on this trunk when
+        it is a member in a QinQ vlan.
+      - The ether-type can be set to any string containing a valid hexadecimal 16 bits
+        number, or any of the well known ether-types; C(0x8100), C(0x9100), C(0x88a8).
+      - This parameter is not supported on Virtual Editions.
+      - You should always wrap this value in quotes to prevent Ansible from interpreting
+        the value as a literal hexadecimal number and converting it to an integer.
+    type: raw
+    version_added: 2.7
   state:
     description:
       - When C(present), ensures that the resource exists.
       - When C(absent), ensures the resource is removed.
-    default: present
+    type: str
     choices:
       - present
       - absent
+    default: present
 extends_documentation_fragment: f5
 author:
   - Tim Rupp (@caphrim007)
+  - Wojciech Wypior (@wojtek0806)
 '''
 
 EXAMPLES = r'''
-- name: Create a ...
+- name: Create a trunk on hardware
   bigip_trunk:
-    name: foo
-    password: secret
-    server: lb.mydomain.com
-    state: present
-    user: admin
+    name: trunk1
+    interfaces:
+      - 1.1
+      - 1.2
+    link_selection_policy: maximum-bandwidth
+    frame_distribution_hash: destination-mac
+    lacp_enabled: yes
+    lacp_mode: passive
+    lacp_timeout: short
+    provider:
+      password: secret
+      server: lb.mydomain.com
+      user: admin
   delegate_to: localhost
 '''
 
 RETURN = r'''
-param1:
-  description: The new param1 value of the resource.
+lacp_mode:
+  description: Operation mode for LACP if the lacp option is enabled for the trunk.
+  returned: changed
+  type: str
+  sample: active
+lacp_timeout:
+  description: Rate at which the system sends the LACP control packets.
+  returned: changed
+  type: str
+  sample: long
+link_selection_policy:
+  description:
+    - LACP policy that the trunk uses to determine which member link (interface)
+      can handle new traffic.
+  returned: changed
+  type: str
+  sample: auto
+frame_distribution_hash:
+  description: Hash that the system uses as the frame distribution algorithm.
+  returned: changed
+  type: str
+  sample: src-dst-ipport
+lacp_enabled:
+  description: Whether the system supports the link aggregation control protocol (LACP) or not.
   returned: changed
   type: bool
-  sample: true
-param2:
-  description: The new param2 value of the resource.
+  sample: yes
+interfaces:
+  description: Interfaces that are part of the trunk.
   returned: changed
-  type: string
-  sample: Foo is bar
+  type: list
+  sample: ['int1', 'int2']
+description:
+  description: Description of the trunk.
+  returned: changed
+  type: str
+  sample: My trunk
+qinq_ethertype:
+  description: Ether-type value used for the packets handled on this trunk when it is a member in a QinQ vlan.
+  returned: changed
+  type: str
+  sample: 0x9100
 '''
 
 from ansible.module_utils.basic import AnsibleModule
-from ansible.module_utils.basic import env_fallback
 
 try:
-    from library.module_utils.network.f5.bigip import HAS_F5SDK
-    from library.module_utils.network.f5.bigip import F5Client
+    from library.module_utils.network.f5.bigip import F5RestClient
     from library.module_utils.network.f5.common import F5ModuleError
     from library.module_utils.network.f5.common import AnsibleF5Parameters
-    from library.module_utils.network.f5.common import cleanup_tokens
-    from library.module_utils.network.f5.common import fq_name
     from library.module_utils.network.f5.common import f5_argument_spec
-    try:
-        from library.module_utils.network.f5.common import iControlUnexpectedHTTPError
-    except ImportError:
-        HAS_F5SDK = False
+    from library.module_utils.network.f5.compare import cmp_simple_list
 except ImportError:
-    from ansible.module_utils.network.f5.bigip import HAS_F5SDK
-    from ansible.module_utils.network.f5.bigip import F5Client
+    from ansible.module_utils.network.f5.bigip import F5RestClient
     from ansible.module_utils.network.f5.common import F5ModuleError
     from ansible.module_utils.network.f5.common import AnsibleF5Parameters
-    from ansible.module_utils.network.f5.common import cleanup_tokens
-    from ansible.module_utils.network.f5.common import fq_name
     from ansible.module_utils.network.f5.common import f5_argument_spec
-    try:
-        from ansible.module_utils.network.f5.common import iControlUnexpectedHTTPError
-    except ImportError:
-        HAS_F5SDK = False
+    from ansible.module_utils.network.f5.compare import cmp_simple_list
 
 
 class Parameters(AnsibleF5Parameters):
@@ -164,7 +215,8 @@ class Parameters(AnsibleF5Parameters):
         'lacpTimeout': 'lacp_timeout',
         'linkSelectPolicy': 'link_selection_policy',
         'distributionHash': 'frame_distribution_hash',
-        'lacp': 'lacp_enabled'
+        'lacp': 'lacp_enabled',
+        'qinqEthertype': 'qinq_ethertype',
     }
 
     api_attributes = [
@@ -174,6 +226,8 @@ class Parameters(AnsibleF5Parameters):
         'linkSelectPolicy',
         'distributionHash',
         'interfaces',
+        'description',
+        'qinqEthertype',
     ]
 
     returnables = [
@@ -182,7 +236,9 @@ class Parameters(AnsibleF5Parameters):
         'link_selection_policy',
         'frame_distribution_hash',
         'lacp_enabled',
-        'interfaces'
+        'interfaces',
+        'description',
+        'qinq_ethertype',
     ]
 
     updatables = [
@@ -191,7 +247,9 @@ class Parameters(AnsibleF5Parameters):
         'link_selection_policy',
         'frame_distribution_hash',
         'lacp_enabled',
-        'interfaces'
+        'interfaces',
+        'description',
+        'qinq_ethertype',
     ]
 
 
@@ -303,22 +361,14 @@ class Difference(object):
 
     @property
     def interfaces(self):
-        if self.want.interfaces is None:
-            return None
-        if self.have.interfaces is None and self.want.interfaces == '':
-            return None
-        if self.have.interfaces is not None and self.want.interfaces == '':
-            return []
-        if self.have.interfaces is None:
-            return self.want.interfaces
-        if set(self.want.interfaces) != set(self.have.interfaces):
-            return self.want.interfaces
+        result = cmp_simple_list(self.want.interfaces, self.have.interfaces)
+        return result
 
 
 class ModuleManager(object):
     def __init__(self, *args, **kwargs):
         self.module = kwargs.get('module', None)
-        self.client = kwargs.get('client', None)
+        self.client = F5RestClient(**self.module.params)
         self.want = ModuleParameters(params=self.module.params)
         self.have = ApiParameters()
         self.changes = UsableChanges()
@@ -360,13 +410,10 @@ class ModuleManager(object):
         result = dict()
         state = self.want.state
 
-        try:
-            if state == "present":
-                changed = self.present()
-            elif state == "absent":
-                changed = self.absent()
-        except iControlUnexpectedHTTPError as e:
-            raise F5ModuleError(str(e))
+        if state == "present":
+            changed = self.present()
+        elif state == "absent":
+            changed = self.absent()
 
         reportable = ReportableChanges(params=self.changes.to_return())
         changes = reportable.to_return()
@@ -389,11 +436,10 @@ class ModuleManager(object):
         else:
             return self.create()
 
-    def exists(self):
-        result = self.client.api.tm.net.trunks.trunk.exists(
-            name=self.want.name
-        )
-        return result
+    def absent(self):
+        if self.exists():
+            return self.remove()
+        return False
 
     def update(self):
         self.have = self.read_current_from_device()
@@ -429,38 +475,87 @@ class ModuleManager(object):
         self.create_on_device()
         return True
 
+    def exists(self):
+        uri = "https://{0}:{1}/mgmt/tm/net/trunk/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
+        )
+        resp = self.client.api.get(uri)
+        try:
+            response = resp.json()
+        except ValueError:
+            return False
+        if resp.status == 404 or 'code' in response and response['code'] == 404:
+            return False
+        return True
+
     def create_on_device(self):
         params = self.changes.api_params()
-        self.client.api.tm.net.trunks.trunk.create(
-            name=self.want.name,
-            **params
+        params['name'] = self.want.name
+        uri = "https://{0}:{1}/mgmt/tm/net/trunk/".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
         )
+        resp = self.client.api.post(uri, json=params)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if 'code' in response and response['code'] in [400, 403, 409]:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
 
     def update_on_device(self):
         params = self.changes.api_params()
-        resource = self.client.api.tm.net.trunks.trunk.load(
-            name=self.want.name
+        uri = "https://{0}:{1}/mgmt/tm/net/trunk/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
         )
-        resource.modify(**params)
+        resp = self.client.api.patch(uri, json=params)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
 
-    def absent(self):
-        if self.exists():
-            return self.remove()
-        return False
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
 
     def remove_from_device(self):
-        resource = self.client.api.tm.net.trunks.trunk.load(
-            name=self.want.name
+        uri = "https://{0}:{1}/mgmt/tm/net/trunk/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
         )
-        if resource:
-            resource.delete()
+        resp = self.client.api.delete(uri)
+        if resp.status == 200:
+            return True
 
     def read_current_from_device(self):
-        resource = self.client.api.tm.net.trunks.trunk.load(
-            name=self.want.name
+        uri = "https://{0}:{1}/mgmt/tm/net/trunk/{2}".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+            self.want.name
         )
-        result = resource.attrs
-        return ApiParameters(params=result)
+        resp = self.client.api.get(uri)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if 'code' in response and response['code'] == 400:
+            if 'message' in response:
+                raise F5ModuleError(response['message'])
+            else:
+                raise F5ModuleError(resp.content)
+        return ApiParameters(params=response)
 
 
 class ArgumentSpec(object):
@@ -478,10 +573,12 @@ class ArgumentSpec(object):
             lacp_enabled=dict(type='bool'),
             lacp_mode=dict(choices=['active', 'passive']),
             lacp_timeout=dict(choices=['short', 'long']),
+            description=dict(),
             state=dict(
                 default='present',
                 choices=['absent', 'present']
-            )
+            ),
+            qinq_ethertype=dict(type='raw'),
         )
         self.argument_spec = {}
         self.argument_spec.update(f5_argument_spec)
@@ -495,17 +592,12 @@ def main():
         argument_spec=spec.argument_spec,
         supports_check_mode=spec.supports_check_mode
     )
-    if not HAS_F5SDK:
-        module.fail_json(msg="The python f5-sdk module is required")
 
     try:
-        client = F5Client(**module.params)
-        mm = ModuleManager(module=module, client=client)
+        mm = ModuleManager(module=module)
         results = mm.exec_module()
-        cleanup_tokens(client)
         module.exit_json(**results)
     except F5ModuleError as ex:
-        cleanup_tokens(client)
         module.fail_json(msg=str(ex))
 
 
