@@ -10,12 +10,12 @@ __metaclass__ = type
 
 ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
-                    'supported_by': 'community'}
+                    'supported_by': 'certified'}
 
 DOCUMENTATION = r'''
 ---
 module: bigiq_regkey_license_assignment
-short_description: Manage regkey license assignment on BIG-IPs from a BIG-IQ.
+short_description: Manage regkey license assignment on BIG-IPs from a BIG-IQ
 description:
   - Manages the assignment of regkey licenses on a BIG-IQ. Assignment means that
     the license is assigned to a BIG-IP, or, it needs to be assigned to a BIG-IP.
@@ -25,10 +25,12 @@ options:
   pool:
     description:
       - The registration key pool to use.
+    type: str
     required: True
   key:
     description:
       - The registration key that you want to assign from the pool.
+    type: str
     required: True
   device:
     description:
@@ -40,6 +42,7 @@ options:
         one device with the same name. BIG-IQ internally recognizes devices by their ID,
         and therefore, this module's cannot guarantee that the correct device will be
         registered. The device returned is the device that will be used.
+    type: str
   managed:
     description:
       - Whether the specified device is a managed or un-managed device.
@@ -49,6 +52,7 @@ options:
     description:
       - Specifies the port of the remote device to connect to.
       - If this parameter is not specified, the default of C(443) will be used.
+    type: int
     default: 443
   device_username:
     description:
@@ -56,19 +60,22 @@ options:
       - This username should be one that has sufficient privileges on the remote device
         to do licensing. Usually this is the C(Administrator) role.
       - When C(managed) is C(no), this parameter is required.
+    type: str
   device_password:
     description:
       - The password of the C(device_username).
       - When C(managed) is C(no), this parameter is required.
+    type: str
   state:
     description:
       - When C(present), ensures that the device is assigned the specified license.
       - When C(absent), ensures the license is revokes from the remote device and freed
         on the BIG-IQ.
-    default: present
+    type: str
     choices:
       - present
       - absent
+    default: present
 extends_documentation_fragment: f5
 author:
   - Tim Rupp (@caphrim007)
@@ -83,34 +90,37 @@ EXAMPLES = r'''
     managed: no
     device_username: admin
     device_password: secret
-    password: secret
-    server: lb.mydomain.com
     state: present
-    user: admin
+    provider:
+      user: admin
+      password: secret
+      server: lb.mydomain.com
   delegate_to: localhost
 
-- name: Register an managed device, by name
+- name: Register a managed device, by name
   bigiq_regkey_license_assignment:
     pool: my-regkey-pool
     key: XXXX-XXXX-XXXX-XXXX-XXXX
     device: bigi1.foo.com
     managed: yes
-    password: secret
-    server: lb.mydomain.com
     state: present
-    user: admin
+    provider:
+      user: admin
+      password: secret
+      server: lb.mydomain.com
   delegate_to: localhost
 
-- name: Register an managed device, by UUID
+- name: Register a managed device, by UUID
   bigiq_regkey_license_assignment:
     pool: my-regkey-pool
     key: XXXX-XXXX-XXXX-XXXX-XXXX
     device: 7141a063-7cf8-423f-9829-9d40599fa3e0
     managed: yes
-    password: secret
-    server: lb.mydomain.com
     state: present
-    user: admin
+    provider:
+      user: admin
+      password: secret
+      server: lb.mydomain.com
   delegate_to: localhost
 '''
 
@@ -119,6 +129,7 @@ RETURN = r'''
 '''
 
 import re
+import time
 
 from ansible.module_utils.basic import AnsibleModule
 
@@ -127,20 +138,13 @@ try:
     from library.module_utils.network.f5.common import F5ModuleError
     from library.module_utils.network.f5.common import AnsibleF5Parameters
     from library.module_utils.network.f5.common import f5_argument_spec
-    from library.module_utils.network.f5.common import exit_json
-    from library.module_utils.network.f5.common import fail_json
+    from library.module_utils.network.f5.ipaddress import is_valid_ip
 except ImportError:
     from ansible.module_utils.network.f5.bigiq import F5RestClient
     from ansible.module_utils.network.f5.common import F5ModuleError
     from ansible.module_utils.network.f5.common import AnsibleF5Parameters
     from ansible.module_utils.network.f5.common import f5_argument_spec
-    from ansible.module_utils.network.f5.common import exit_json
-    from ansible.module_utils.network.f5.common import fail_json
-try:
-    import netaddr
-    HAS_NETADDR = True
-except ImportError:
-    HAS_NETADDR = False
+    from ansible.module_utils.network.f5.ipaddress import is_valid_ip
 
 
 class Parameters(AnsibleF5Parameters):
@@ -205,11 +209,9 @@ class ModuleParameters(Parameters):
 
     @property
     def device_is_address(self):
-        try:
-            netaddr.IPAddress(self.device)
+        if is_valid_ip(self.device):
             return True
-        except (ValueError, netaddr.core.AddrFormatError):
-            return False
+        return False
 
     @property
     def device_is_id(self):
@@ -395,7 +397,7 @@ class Difference(object):
 class ModuleManager(object):
     def __init__(self, *args, **kwargs):
         self.module = kwargs.get('module', None)
-        self.client = kwargs.get('client', None)
+        self.client = F5RestClient(**self.module.params)
         self.want = ModuleParameters(params=self.module.params, client=self.client)
         self.have = ApiParameters()
         self.changes = UsableChanges()
@@ -484,6 +486,10 @@ class ModuleManager(object):
         self.remove_from_device()
         if self.exists():
             raise F5ModuleError("Failed to delete the resource.")
+        # Artificial sleeping to wait for remote licensing (on BIG-IP) to complete
+        #
+        # This should be something that BIG-IQ can do natively in 6.1-ish time.
+        time.sleep(60)
         return True
 
     def create(self):
@@ -505,6 +511,11 @@ class ModuleManager(object):
                 "Failed to license the remote device."
             )
         self.wait_for_device_to_be_licensed()
+
+        # Artificial sleeping to wait for remote licensing (on BIG-IP) to complete
+        #
+        # This should be something that BIG-IQ can do natively in 6.1-ish time.
+        time.sleep(60)
         return True
 
     def create_on_device(self):
@@ -530,7 +541,7 @@ class ModuleManager(object):
             if 'message' in response:
                 raise F5ModuleError(response['message'])
             else:
-                raise F5ModuleError(resp._content)
+                raise F5ModuleError(resp.content)
 
     def wait_for_device_to_be_licensed(self):
         count = 0
@@ -552,7 +563,7 @@ class ModuleManager(object):
                 if 'message' in response:
                     raise F5ModuleError(response['message'])
                 else:
-                    raise F5ModuleError(resp._content)
+                    raise F5ModuleError(resp.content)
             if response['status'] == 'LICENSED':
                 count += 1
             else:
@@ -611,16 +622,13 @@ def main():
         supports_check_mode=spec.supports_check_mode,
         required_if=spec.required_if
     )
-    if not HAS_NETADDR:
-        module.fail_json(msg="The python netaddr module is required")
 
     try:
-        client = F5RestClient(module=module)
-        mm = ModuleManager(module=module, client=client)
+        mm = ModuleManager(module=module)
         results = mm.exec_module()
-        exit_json(module, results, client)
+        module.exit_json(**results)
     except F5ModuleError as ex:
-        fail_json(module, ex, client)
+        module.fail_json(msg=str(ex))
 
 
 if __name__ == '__main__':
